/* ========================================================================= */
/* VARIABLES =============================================================== */
/* ========================================================================= */

var pointsOfInterest = {
	informationPoint: {
		img: 'icon-information-point@2x.png',
		text: 'Punto de información'
	},
	singularLandscapes: {
		img: 'icon-singular-landscapes@2x.png',
		text: 'Paisajes singulares'
	},
	serviceCenter: {
		img: 'icon-service-center@2x.png',
		text: 'Centro de Servicios'
	},
	toilets: {
		img: 'icon-toilets@2x.png',
		text: 'Aseos'
	},
	campingZone: {
		img: 'icon-camping-zone@2x.png',
		text: 'Zona de acampada'
	},
	refuge: {
		img: 'icon-refuge@2x.png',
		text: 'Refugio'
	},
	visitorCenter: {
		img: 'icon-visitor-center@2x.png',
		text: 'Centro del visitante'
	},
	exhibition: {
		img: 'icon-exhibition@2x.png',
		text: 'Sala de exposiciones'
	},
	projection: {
		img: 'icon-projection@2x.png',
		text: 'Proyección'
	},
	nationalParador: {
		img: 'icon-national-parador@2x.png',
		text: 'Parador Nacional'
	},
	cableRailway: {
		img: 'icon-cable-railway@2x.png',
		text: 'Teleférico'
	}
};

/* ========================================================================= */
/* HELPERS ================================================================= */
/* ========================================================================= */

getDevice = function() {
	return (
		window.getComputedStyle(
	    	document.querySelector('html'), ':before'
		).getPropertyValue('content')
		.replace(/["']/g, '') // Fix for: "'mobile'", ""mobile""...
	);
};

trimInput = function(value) {
	return value.replace(/^\s*|\s*$/g, '');
};

/* ========================================================================= */
/* FUNCTIONS =============================================================== */
/* ========================================================================= */
var siteHandler = {
	/**
	 * Carga un template con los datos que recibe como parámetro.
	 *
	 * @param  {String}   name    	- Nombre del fichero .hbs donde se encuentra el template
	 * @param  {String}   where   	- Selector CSS donde vamos a insertar el template
	 * @param  {Object}	  context	- Objeto con los datos que va a utilizar el template
	 * @param  {Boolean}  append  	- Si es true inserta el template al final de un contenedor para no borrar su contenido
	 * @param  {Function} callback  - Función de callback
	 */
	loadTemplate: function(name, where, context, append, func) {
		var location = where || '#' + name,
			data = context,
			template = JST['app/templates/' + name + '.hbs'],
			html = template(data),
			callback = func;

		if (append)
			$(location).append(html);
		else
			document.querySelector(location).innerHTML = html;
		if (func)
			callback();
	},
	/**
	 * Devuelve la ruta a la imagen del punto de interés que recibe según los parámetros
	 *
	 * @param  {String} type  - Tipo del puto de interés. Definido en el objeto pointsOfInterest
	 * @return {String}       - Ruta de la imagen
	 */
	getPointIcon: function(type) {
		var path = 'images/content/';

		return path + pointsOfInterest[type].img;
	},
	/**
	 * Devuelve el texto del punto de interés que recibe según el nombre
	 *
	 * @param  {String} type  - Tipo del puto de interés. Definido en el objeto pointsOfInterest
	 * @return {String}       - Texto del punto de interés
	 */
	getPointText: function(type) {
		return pointsOfInterest[type].text;
	},
	/**
	 * Establece los eventos de la app
	 */
	setEvents: {
		// activamos la pantalla principal (caracteristicas + ficha)
		activePrincipal: function(e){

			// reiniciamos eventos
			$('.area').off('click');

			// iterate through area objects to asign click event for modals
			_.each($('.area'), function(area, index) {
				 $(area).off('click').on('click', function(event) {
				 	event.preventDefault();
			 		siteHandler.loadTemplate($(area).data('area'),'#nucleusModal .modal-body');
				 	$('#nucleusModal').modal();
				 });
			});

			$('.btn-1').off('click').on('click', function(){
				var $graphs = $('.graph-wrapper'),
				    $elements = $('.s-e'),
				    $text = $('.graph-text');
				//se añade la clase playing para "bloquear" el evento.
				if (!$('.btn-1').hasClass('playing')){
					$('.btn-1 .fa').toggleClass('fa-plus-circle fa-minus-circle');
					$('.btn-1').addClass('playing');
					$graphs.addClass('animated fadeIn').show();
					$text.addClass('animated fadeIn').show();
					//ocultamos todos los elementos del size (size-elements + size-labels)
					$elements.hide();

					_.each($elements, function(num, index, list){
						// a cada elemento de la gráfica le asignamos un timeOut para mostrarlos uno a uno
						setTimeout(function() {
						    $('.graph-element-'+(index+1)).addClass('animated fadeIn').show();
						}, 1000*(index+1));
					});

				}
				else {
					$graphs.removeClass('fadeIn').addClass('fadeOut');
					$text.removeClass('fadeIn').addClass('fadeOut');
					setTimeout(function() {
						$text.hide().removeClass('fadeOut');
						$graphs.animate({height: "0px"}, 500, function(){
							$graphs.hide().removeClass('fadeOut').css({'height':'auto'});
						})
						$('.btn-1').removeClass('playing');
						$('.btn-1 .fa').toggleClass('fa-plus-circle fa-minus-circle');
					}, 700);
				}
			});

			$('.btn-2').off('click').on('click', function(){
				var $graphs = $('.graph-text-special');

				//se añade la clase playing para "bloquear" el evento.
				if (!$('.btn-2').hasClass('playing')){
					$('.btn-2 .fa').toggleClass('fa-plus-circle fa-minus-circle');
					$('.btn-2').addClass('playing');
					$graphs.addClass('animated fadeIn').show();
				}
				else {
					$graphs.removeClass('fadeIn').addClass('fadeOut');
					setTimeout(function() {
						$graphs.animate({height: "0px"}, 500, function(){
							$graphs.hide().removeClass('fadeOut').css({'height':'auto'});
						})
						$('.btn-2').removeClass('playing');
						$('.btn-2 .fa').toggleClass('fa-plus-circle fa-minus-circle');
					}, 700);
				}
			});

			// $('.section-icon').off('click').on('click', function() {
			// 	var $parent = $(this).parent();
			// 		$content = $parent.find('.section-content');
			// 		$graphs = $parent.find('.section-graph');

			// 	if (! $parent.hasClass('active')) {
			// 		$parent.addClass('active');
			// 		$content.addClass('fadeIn');
			// 		_.each($graphs, function (graph) {
			// 			switch	(graph.id) {
			// 				case 'ageMaleIsland':
			// 					ageMaleIslandChart = c3.generate(data[graph.id]);
			// 					break;
			// 				case 'ageFemaleIsland':
			// 					ageFemaleIslandChart = c3.generate(data[graph.id]);
			// 					break;
			// 				case 'ageMaleYear':
			// 					ageMaleYearChart = c3.generate(data[graph.id]);
			// 					break;
			// 				case 'ageFemaleYear':
			// 					ageFemaleYearChart = c3.generate(data[graph.id]);
			// 					break;
			// 				default:
			// 					c3.generate(data[graph.id]);
			// 			}
			// 		});
			// 	}
			// 	else {
			// 		$parent.removeClass('active');
			// 		$content.removeClass('fadeIn');
			// 	}

			// });

			$('.island-zone').off('click').on('click', function(event) {
				/* Act on the event */
				$marker = $(this).find('.data-marker');

				// if is not visible
				if (!$marker.is(':visible')) {
					$marker.addClass('animated fadeInDown').show();
				}
				else {
					$marker.hide();
				}
			});

			$('.info-pic').off('click').on('click', function(event) {
				event.preventDefault();
				/* Act on the event */
				var $parent = $(event.currentTarget).parent(),
					$p = $parent.find('p'),
					$header = $parent.find('h3');

				if ($p.is(':visible')){
					$p.hide().removeClass('fadeIn');
					$header.hide().removeClass('fadeIn');
				}
				else {
					$p.addClass('fadeIn').show();
					$header.addClass('fadeIn').show();
				}
			});

			$('.section-icon').off('click').on('click', function(){
				var $this = $(this),
					$container = $( $this.parent() ), // Note: set CSS → height: 0;
					$elements = $container.find('.section-title, .section-element'), // Note: set CSS → opacity: 0
					$separator = $container.find('.separator'); // Note: set CSS → display: none

				// Se añade la clase playing para "bloquear" el evento.
				// Si la animación no está hecha...
				if ( !$this.hasClass('playing') && !$this.hasClass('done') ) {
					$this.find('.fa').toggleClass('fa-plus-circle fa-minus-circle');
					$this.addClass('playing');

					// Expande el contenedor
					$container.addClass('active');
					// Muestra el separador
					$separator.addClass('animated fadeInLeftBig').show();

					// Ocultamos los elementos
					_.each($elements, function (element, index, list) {
						// Mostramos los elementos uno a uno
						$(element).delay(600*(index+1)).animate({ opacity: 1}, function() {});

						// Cuando termine marcamos la animación como hecha
						if (index == list.length -1) {
							setTimeout(function() {
								$this.removeClass('playing').addClass('done');
							}, 600*(index+2));
						}
					});
				}
				// Si la animación está hecha...
				else if ( !$this.hasClass('playing') && $this.hasClass('done') ) {
					// Ocultamos los elementos
					$elements.animate({ opacity: 0}, function() {});

					// Después de que se oculten los elementos contraemos el contenedor y desbloqueamos el botón
					setTimeout(function() {
						$this.removeClass('done');
						$separator.removeClass('animated fadeInLeftBig').hide();
						$container.removeClass('active');

						$this.removeClass('playing');
						$this.find('.fa').toggleClass('fa-plus-circle fa-minus-circle');
					}, 600);
				}
			});

			$('.curtain-arrow').off('click').on('click', function() {
				$(this).find('i').toggleClass('fa-angle-down fa-angle-up');
				$(this).parent().find('.curtain-content').toggleClass('active');
			});
		},
		activeLegend: function() {
			$('#curtainIcon').on('click', function(event) {
				event.preventDefault();
				$(this).find('i').toggleClass('fa-search-plus fa-search-minus');
				/* Act on the event */
				$('.legend-wrapper').toggleClass('closed');
			});
			$('#infoIcon').on('click', function(event) {
		        event.preventDefault();
		        event.stopPropagation();
		        /* Act on the event */
		        siteHandler.loadTemplate($(this).data('modal'),'#nucleusModal .modal-body');
		        $('#nucleusModal').modal();
	       });
		},
		centerModals: function(){
			function modalCenter(){
				$('.modal').each(function(i){
					$clone = $(this).clone().css('display', 'block').appendTo('body');
					var top = Math.round(($clone.height() - $clone.find('.modal-content').height()) / 2);
					top = top > 0 ? top : 0;
					$clone.remove();
					$(this).find('.modal-content').css("margin-top", top);
				});
			};
			$('.modal').on('show.bs.modal', modalCenter);
			$(window).on('resize', modalCenter);
		},
		activeMapLegend: function() {
			$('.map-button').on('click', function() {
				var $parent = $(this).parent();
				$parent.toggleClass('expanded');
				$(this).find('.fa').toggleClass('fa-angle-double-left fa-angle-double-right');
			});
		},
		resize: function() {
			lastDevice = getDevice();
			window.addEventListener('resize', _.debounce(function() {
				var currentDevice = getDevice();

				if (currentDevice != lastDevice) {
					lastDevice = currentDevice;

					if (currentDevice == 'mobile') {
						map.setView([28.654, -17.861], 10);
						map.minZoom = 6;
						map.setZoom(10);
					}
					else if (getDevice() == 'tablet') {
						map.setView([28.711, -17.869], 12);
						map.minZoom = 7;
						map.setZoom(12);
					}
					else {
						map.setView([28.711, -17.869], 12);
						map.minZoom = 7;
						map.setZoom(12);
					}
				}
			}, 300));
		}
	},
	init: function() {
		//load template layout (header, grid, and site-canvas)
		this.loadTemplate('layout', '#mainWrapper', null, false, function(){
			siteHandler.loadTemplate('main');
			siteHandler.setEvents.activePrincipal();
			siteHandler.setEvents.activeLegend();
			siteHandler.setEvents.centerModals();
			siteHandler.setEvents.activeMapLegend();
			siteHandler.setEvents.resize();
		});

		L.mapbox.accessToken = 'pk.eyJ1Ijoiam9zZW5tYXIiLCJhIjoiRTBnY2plZyJ9.16b9pM_DfVdunAs6jZmE-A';

		var southWest = L.latLng(28.445, -18.169),
		    northEast = L.latLng(28.864, -17.618),
		    bounds = L.latLngBounds(southWest, northEast);

		if (getDevice() == 'mobile') {
			map = L.mapbox.map('map', 'josenmar.83a092d0', {
				maxBounds: bounds,
			    maxZoom: 14,
			    minZoom: 6
			}).setView([28.654, -17.861], 10);
			map.setZoom(10);
		}
		else if (getDevice() == 'tablet'){
			map = L.mapbox.map('map', 'josenmar.83a092d0', {
				maxBounds: bounds,
			    maxZoom: 14,
			    minZoom: 7
			}).setView([28.711, -17.869], 12);
			map.setZoom(12);
		}
		else {
			map = L.mapbox.map('map', 'josenmar.83a092d0', {
				maxBounds: bounds,
			    maxZoom: 14,
			    minZoom: 7
			}).setView([28.711, -17.869], 12);
			map.setZoom(12);
		}

		// // Disable drag and zoom handlers.
		// map.dragging.disable();
		// map.touchZoom.disable();
		// map.doubleClickZoom.disable();
		map.scrollWheelZoom.disable();

		// // Disable tap handler, if present.
		// if (map.tap) map.tap.disable();

		var layers = document.getElementById('menu-ui'),
			popup = new L.Popup({ autoPan: false });

		function addLayer(layer, name, zIndex) {
			layer
				.setZIndex(zIndex)
				.addTo(map);

		    // Create a simple layer switcher that
		    // toggles layers on and off.
		    var polygon = $('.'+name);
		    polygon.on('click', function(e){
		    	e.preventDefault();
		    	e.stopPropagation();

		    	if (map.hasLayer(layer)) {
		    		map.removeLayer(layer);
		     		// this.className = '';
		        } else {
		    		map.addLayer(layer);
				// this.className = 'active';
				}
			});

		    // layers.appendChild(link);
		}

		var myDataLayer = L.mapbox.featureLayer().loadURL('js/data/caldera-taburiente-data.json');
		var legendContainer = $('.map-legend');
		legendContainer.append('<h2>Rutas</h2><ul></ul>');
		// Set a custom icon on each marker based on feature properties.
		myDataLayer.on('layeradd', function(e) {
			var layer = e.layer;
			// Style points or lines
			if (layer.feature.geometry.type == 'Point') {
				// $('.map-legend ul').append('<li class="clearfix '+e.layer.feature.properties.id+'"><div class="svg-wrapper"></div><span>'+e.layer.feature.properties.name+'</span></li>');
				layer.setIcon(getStyle(layer.feature));
			}
			else {
				$('.map-legend ul').append('<li class="clearfix '+e.layer.feature.properties.id+'"><div class="svg-wrapper">'+getSVG(e.layer.feature.properties.id)+'</div><span>'+e.layer.feature.properties.name+'</span></li>');
				layer.setStyle(getStyle(layer.feature));
			}
			onEachFeature: onEachFeature(layer.feature, layer);
		});

		addLayer(myDataLayer, 'data', 2);

		function getSVG(id) {
			return '<svg version="1.1" id="Capa_1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px"viewBox="0 0 23.1 13.2" enable-background="new 0 0 23.1 13.2" xml:space="preserve"> <path fill="none" stroke="'+getColor(id)+'" stroke-width="2" stroke-miterlimit="10" d="M3.9,10.5c0,0,5.5,1.4,7.1-3.3 c1.7-4.9,5.5-4.3,8.4-4.3"/> <circle fill="none" stroke="'+getColor(id)+'" stroke-width="2" stroke-miterlimit="10" cx="2.9" cy="10.4" r="1.4"/> <circle fill="none" stroke="'+getColor(id)+'" stroke-width="2" stroke-miterlimit="10" cx="20.3" cy="2.8" r="1.4"/> </svg>';
		}

		function getStyle(feature) {
			if (feature.geometry.type == 'Polygon') {
				return {
					weight: 0,
					opacity: 1,
					color: '#fff',
					fillOpacity: 0.6,
					fillColor: '#e78124'
				};
			}
			else if (feature.geometry.type == 'LineString') {
				return {
					weight: 2, 		// Stroke width in pixels
					// dashArray: "10, 6"	// Dashed line
					color: getColor(feature.properties.id, feature.geometry.type)
				};
			}
			else if (feature.geometry.type == 'Point') {
				var name = feature.properties.name;

				return L.mapbox.marker.icon({
					'marker-color': getColor(feature.properties.id, feature.geometry.type)
				});
			}
		}

		function getStyleMobile(feature) {
			if (feature.geometry.type == 'Polygon') {
				return {
					weight: 0.3,
					opacity: 1,
					color: '#fff',
					fillOpacity: 1,
					fillColor: '#e78124'
				};
			}
		}

		// get color depending on enp id
		function getColor(id, type) {
			if (type == 'LineString'  || type === undefined) {
				return (id == 'l0')  ? '#861759' : // Roque de Los Muchachos - Torre de vigilancia del Time (#861759)
					   (id == 'l1')  ? '#000' : // Roque de Los Muchachos - Espigón del Roque (#AD0E00)
					   (id == 'l3')  ? '#0A4451' : // Pista del Pico de La Nieve - Roque de Los Muchachos (#0A4451)
					   (id == 'l4')  ? '#656C23' : // Pista del Pico de La Nieve - Ermita de la Virgen del Pino (#00825D)
					   (id == 'l5')  ? '#FFD75A' : // Los Brecitos-Zona de Acampada (#FFCC00)
					   (id == 'l6')  ? '#000' : // Zona de Acampada - Hoyo Verde (#000982)
					   (id == 'l7')  ? '#02B1BD' : // Zona de Acampada - Barranco de las Angustias (#00ADB0)
					   (id == 'l8')  ? '#0C6064' : // Cumbrecita - Zona de Acampada (#004B96)
					   (id == 'l9')  ? '#D87C0E' : // La Cumbrecita - Lomo de las Chozas - Mirador de los Roques (#FF6600)
					   (id == 'l10') ? '#000' : // Lomo de las Chozas - La Cancelita (#480080)
					   (id == 'l11') ? '#8DA42B' : // Cumbrecita - Bejenado (#00944A)
					   (id == 'l12') ? '#000' : // Pista de Valencia - Pico Bejenado (alternativa 2) (#00CC00)
					   (id == 'l13') ? '#D83C45' : // Pista de Valencia-Pico Bejenado (#FA3411)
					   (id == 'l14') ? '#892C36' : // Cumbrecita-Ermita de la Virgen del Pino (#821100)
						'#000';
			}
			else if (type == 'Point') {
				return (id == 'p0') ? '#f3981d' : // Centro de visitantes P.N. Caldera de Taburiente
					   (id == 'p1') ? '#8ca22b' : // Refugio Punta de Los Roques
					   (id == 'p2') ? '#02b1bd' : // Zona de acampada
					   (id == 'p3') ? '#d83c45' : // Roque de los Muchachos
						'';
			}
		}

		function onEachFeature(feature, layer) {
			layer.on({
				click: showTooltip
			});
		}

		var closeTooltip;

		function showTooltip(e) {
			var layer        = e.target,
				propiedades  = '',
				html         = '',
				currentPlace = '';

			if (layer.feature.geometry.type == "Point") {
				//limpiamos un poco los datos de layer.feature.properties
				propiedades = _.pick(layer.feature.properties, 'name', 'pointsOfInterest', 'id');

				currentPlace = propiedades.name;
				var currentIcons = propiedades.pointsOfInterest;

				html = '<div class="marker-title">'+currentPlace+'</div>';
				html += '<div class="icon-wrapper clearfix">';

				//insert each icon
				_.each(currentIcons, function(val, i){
					html += '<div class="icon-point retina"><img src="'+ siteHandler.getPointIcon(val) +'"><span>'+ siteHandler.getPointText(val) +'</span></div>';
				});
				html += '</div>';
			} else {
				//limpiamos un poco los datos de layer.feature.properties
				propiedades = _.pick(layer.feature.properties, 'name', 'description');

				currentPlace = propiedades.name;
				var currentDesc = propiedades.description;

				html = '<div class="marker-title">'+currentPlace+'</div>';
				html += '<div class="desc-wrapper clearfix">'+currentDesc+'</div>';
			}

			popup.setLatLng(e.latlng);
			popup.setContent(html);

			if (!popup._map)
				popup.openOn(map);

			//set the color for the icons
			if (layer.feature.geometry.type == "Point") {
				$('.icon-point').css({
					'background-color' : getColor(propiedades.id, layer.feature.geometry.type)
				});
			}

			window.clearTimeout(closeTooltip);
		}
	}
};

siteHandler.init();
